#ifndef BRICK_COMPOSITE_H
#define BRICK_COMPOSITE_H

#include "brick_emits.h"
#include "brick_math.h"
#include "brick_logic.h"
#include "brick_digital.h"
#include "brick_link.h"
#include "brick_interface.h"
#include "brick_statistic.h"

void buildTree(PIVector<BrickBase * > & bricks, PIVector<PIVector<BrickBase * > > & tree);
PIVector<PIVector<BrickBase * > > buildSubtree(PIVector<BrickBase * > & bricks, PIVector<PIVector<BrickBase * > > & tree);
void sumTrees(PIVector<PIVector<BrickBase * > > & ft, PIVector<PIVector<BrickBase * > > & st);


class BrickCompositeBase: public BrickBase {
	public: BrickCompositeBase(int inputs_num = 0, int outputs_num = 0, int parameters_num = 1): BrickBase(inputs_num, outputs_num, parameters_num) {lib = "Composite";}
};

class BrickCompositeInput;
class BrickCompositeOutput;
class BrickCompositeParameters;
class BrickCompositeInformation;

class BrickComposite: public BrickCompositeBase
{
	friend class BrickManager;
public:
	BrickComposite();
	virtual BrickComposite * copy() {BrickComposite * t = new BrickComposite(*this); t->parameterChanged(0); return t;}
	virtual bool tick_body(double time);
	Mode mode() const {return mode_;}
	void setMode(Mode m) {mode_ = m;}
	void buildBrickTree() {buildTree(bricks, tree);}
	void clear();
	static BrickBase * findBaseBrick(const PIString & code_name);
	
protected:
	virtual void parameterChanged(int index);
	virtual void started() {parameterChanged(0); for (uint i = 0; i < bricks.size(); ++i) {bricks[i]->setManager(manager_); bricks[i]->started();}}
	virtual void finished() {for (uint i = 0; i < bricks.size(); ++i) bricks[i]->finished();}
	virtual void saveInputsToDefault() {defInputs = inputs; for (uint i = 0; i < bricks.size(); ++i) bricks[i]->saveInputsToDefault();}
	virtual void resetOutputs() {for (uint i = 0; i < bricks.size(); ++i) bricks[i]->resetOutputs();}
	virtual void reset_specified() {for (uint i = 0; i < bricks.size(); ++i) bricks[i]->reset_specified();}
	virtual void reset() {BrickBase::reset(); for (uint i = 0; i < bricks.size(); ++i) bricks[i]->reset();}
	virtual void proceedConnections(bool compositeToo = false);

	BrickBase * findBrick(const PIString & name);
	static void loadBrick(PIConfig & sr, const PIString & prefix, BrickBase * b, BrickManager * m = 0);
	static void fillBaseBricks();
	
	static PIVector<BrickBase * > baseBricks;
	PIVector<BrickBase * > bricks;
	PIVector<PIVector<BrickBase * > > tree;
	PIVector<BrickBase * > * tl;
	BrickCompositeInput * bi;
	BrickCompositeOutput * bo;
	BrickCompositeParameters * bp;
	BrickCompositeInformation * inf;
	Mode mode_;
	int header_;
	
};


class BrickCompositeInput: public BrickCompositeBase
{
	friend class BrickManager;
public:
	BrickCompositeInput(int inputs_num = 1);
	virtual BrickCompositeInput * copy() {return new BrickCompositeInput(*this);}
	virtual bool tick_body(double time) {return true;}
	virtual void parameterChanged(int index) {if (outNames.size_s() > index) outNames[index] = parameters[index].toString();}
	virtual void outCountChanged(int count) {setParametersCount(count, "");}
};


class BrickCompositeOutput: public BrickCompositeBase
{
	friend class BrickManager;
public:
	BrickCompositeOutput(int outputs_num = 1);
	virtual BrickCompositeOutput * copy() {return new BrickCompositeOutput(*this);}
	virtual bool tick_body(double time) {return true;}
	virtual void parameterChanged(int index) {if (inNames.size_s() > index) inNames[index] = parameters[index].toString();}
	virtual void inCountChanged(int count) {setParametersCount(count, "");}
};


class BrickCompositeParameters: public BrickCompositeBase
{
	friend class BrickManager;
public:
	BrickCompositeParameters();
	virtual BrickCompositeParameters * copy() {return new BrickCompositeParameters(*this);}
	virtual void outCountChanged(int count) {setParametersCount(count, "p");}
	virtual void parameterChanged(int index);
};


class BrickCompositeInformation: public BrickCompositeBase
{
	friend class BrickManager;
public:
	BrickCompositeInformation();
	virtual BrickCompositeInformation * copy() {return new BrickCompositeInformation(*this);}
};

#endif // BRICK_COMPOSITE_H
