#ifndef BRICK_DIGITAL_H
#define BRICK_DIGITAL_H

#include "brick_base.h"


class BrickDigitalBase: public BrickBase {
public: BrickDigitalBase(int inputs_num = 1, int outputs_num = 1, int parameters_num = 0): BrickBase(inputs_num, outputs_num, parameters_num) {lib = "Digital"; inNames[0] = outNames[0] = "0";}
};


class BrickDigitalTriggerRS: public BrickDigitalBase {
public:
	BrickDigitalTriggerRS(): BrickDigitalBase(2, 2) {
		codeName_ = "BrickDigitalTriggerRS";
		type = name_ = "RStrigger";
		inNames[0] = "S";
		inNames[1] = "R";
		outNames[0] = "T";
		outNames[1] = "To";
	}
	enum Inputs {S, R};
	enum Outputs {T, To};
	virtual BrickBase * copy() {return new BrickDigitalTriggerRS(*this);}
	virtual bool tick_body(double time);
};


class BrickDigitalTriggerD: public BrickDigitalBase {
public:
	BrickDigitalTriggerD(): BrickDigitalBase(2, 2) {
		codeName_ = "BrickDigitalTriggerD";
		type = name_ = "Dtrigger";
		inNames[0] = "D";
		inNames[1] = "C";
		outNames[0] = "T";
		outNames[1] = "To";
	}
	enum Inputs {D, C};
	enum Outputs {T, To};
	virtual BrickBase * copy() {return new BrickDigitalTriggerD(*this);}
	virtual bool tick_body(double time);
};


class BrickDigitalTriggerJK: public BrickDigitalBase {
public:
	BrickDigitalTriggerJK(): BrickDigitalBase(3, 2) {
		codeName_ = "BrickDigitalTriggerJK";
		type = name_ = "JKtrigger";
		inNames[0] = "J";
		inNames[1] = "C";
		inNames[2] = "K";
		outNames[0] = "T";
		outNames[1] = "To";
	}
	enum Inputs {J, C, K};
	enum Outputs {T, To};
	virtual BrickBase * copy() {return new BrickDigitalTriggerJK(*this);}
	virtual void reset_specified() {pc = 0.;}
	virtual bool tick_body(double time);
private:
	double pc;
};


class BrickDigitalCounter2: public BrickDigitalBase {
public:
	BrickDigitalCounter2(): BrickDigitalBase(2, 2) {codeName_ = "BrickDigitalCounter2"; type = name_ = "Counter2"; inNames[0] = "/ C"; inNames[1] = "R"; v = 0.;}
	enum Inputs {C, R};
	virtual BrickBase * copy() {return new BrickDigitalCounter2(*this);}
	virtual void reset_specified() {v = dv = 0.;}
	virtual bool tick_body(double time);
private:
	double v, dv;
};


class BrickDigitalCounter4: public BrickDigitalBase {
public:
	BrickDigitalCounter4(): BrickDigitalBase(2, 4) {codeName_ = "BrickDigitalCounter4"; type = name_ = "Counter4"; inNames[0] = "/ C"; inNames[1] = "R"; v = 0.;}
	enum Inputs {C, R};
	virtual BrickBase * copy() {return new BrickDigitalCounter4(*this);}
	virtual bool tick_body(double time);
private:
	double v, dv;
};


class BrickDigitalCounter8: public BrickDigitalBase {
public:
	BrickDigitalCounter8(): BrickDigitalBase(2, 8) {codeName_ = "BrickDigitalCounter8"; type = name_ = "Counter8"; inNames[0] = "/ C"; inNames[1] = "R"; v = 0.;}
	enum Inputs {C, R};
	virtual BrickBase * copy() {return new BrickDigitalCounter8(*this);}
	virtual bool tick_body(double time);
private:
	double v, dv;
};


class BrickDigitalCoder1: public BrickDigitalBase {
public:
	BrickDigitalCoder1(): BrickDigitalBase(2, 1) {codeName_ = "BrickDigitalCoder1"; type = name_ = "Coder1";}
	virtual BrickBase * copy() {return new BrickDigitalCoder1(*this);}
	virtual bool tick_body(double time) {if (inputs[1] > 0.) outputs[0] = 1.; else outputs[0] = 0.; return true;}
};


class BrickDigitalCoder2: public BrickDigitalBase {
public:
	BrickDigitalCoder2(): BrickDigitalBase(4, 2) {codeName_ = "BrickDigitalCoder2"; type = name_ = "Coder2";}
	virtual BrickBase * copy() {return new BrickDigitalCoder2(*this);}
	virtual bool tick_body(double time);
};


class BrickDigitalCoder3: public BrickDigitalBase {
public:
	BrickDigitalCoder3(): BrickDigitalBase(8, 3) {codeName_ = "BrickDigitalCoder3"; type = name_ = "Coder3";}
	virtual BrickBase * copy() {return new BrickDigitalCoder3(*this);}
	virtual bool tick_body(double time);
};


class BrickDigitalCoder4: public BrickDigitalBase {
public:
	BrickDigitalCoder4(): BrickDigitalBase(16, 4) {codeName_ = "BrickDigitalCoder4"; type = name_ = "Coder4";}
	virtual BrickBase * copy() {return new BrickDigitalCoder4(*this);}
	virtual bool tick_body(double time);
};


class BrickDigitalDecoder1: public BrickDigitalBase {
public:
	BrickDigitalDecoder1(): BrickDigitalBase(1, 2) {codeName_ = "BrickDigitalDecoder1"; type = name_ = "Decoder1";}
	virtual BrickBase * copy() {return new BrickDigitalDecoder1(*this);}
	virtual bool tick_body(double time);
};


class BrickDigitalDecoder2: public BrickDigitalBase {
public:
	BrickDigitalDecoder2(): BrickDigitalBase(2, 4) {codeName_ = "BrickDigitalDecoder2"; type = name_ = "Decoder2";}
	virtual BrickBase * copy() {return new BrickDigitalDecoder2(*this);}
	virtual bool tick_body(double time);
};


class BrickDigitalDecoder3: public BrickDigitalBase {
public:
	BrickDigitalDecoder3(): BrickDigitalBase(3, 8) {codeName_ = "BrickDigitalDecoder3"; type = name_ = "Decoder3";}
	virtual BrickBase * copy() {return new BrickDigitalDecoder3(*this);}
	virtual bool tick_body(double time);
};


class BrickDigitalDecoder4: public BrickDigitalBase {
public:
	BrickDigitalDecoder4(): BrickDigitalBase(4, 16) {codeName_ = "BrickDigitalDecoder4"; type = name_ = "Decoder4";}
	virtual BrickBase * copy() {return new BrickDigitalDecoder4(*this);}
	virtual bool tick_body(double time);
};


class BrickDigitalMux1: public BrickDigitalBase {
public:
	BrickDigitalMux1(): BrickDigitalBase(3, 1) {codeName_ = "BrickDigitalMux1"; type = name_ = "Mux1"; inNames[2] = "A0";}
	virtual BrickBase * copy() {return new BrickDigitalMux1(*this);}
	virtual bool tick_body(double time);
};


class BrickDigitalMux2: public BrickDigitalBase {
public:
	BrickDigitalMux2(): BrickDigitalBase(6, 1) {codeName_ = "BrickDigitalMux2"; type = name_ = "Mux2"; inNames[4] = "A0"; inNames[5] = "A1";}
	virtual BrickBase * copy() {return new BrickDigitalMux2(*this);}
	virtual bool tick_body(double time);
};


class BrickDigitalMux3: public BrickDigitalBase {
public:
	BrickDigitalMux3(): BrickDigitalBase(11, 1) {codeName_ = "BrickDigitalMux3"; type = name_ = "Mux3"; inNames[8] = "A0"; inNames[9] = "A1"; inNames[10] = "A2";}
	virtual BrickBase * copy() {return new BrickDigitalMux3(*this);}
	virtual bool tick_body(double time);
};


class BrickDigitalMux4: public BrickDigitalBase {
public:
	BrickDigitalMux4(): BrickDigitalBase(20, 1) {codeName_ = "BrickDigitalMux4"; type = name_ = "Mux4"; inNames[16] = "A0"; inNames[17] = "A1"; inNames[18] = "A2"; inNames[19] = "A3";}
	virtual BrickBase * copy() {return new BrickDigitalMux4(*this);}
	virtual bool tick_body(double time);
};


class BrickDigitalDemux1: public BrickDigitalBase {
public:
	BrickDigitalDemux1(): BrickDigitalBase(2, 2) {codeName_ = "BrickDigitalDemux1"; type = name_ = "Demux1"; inNames[1] = "A0";}
	virtual BrickBase * copy() {return new BrickDigitalDemux1(*this);}
	virtual bool tick_body(double time);
};


class BrickDigitalDemux2: public BrickDigitalBase {
public:
	BrickDigitalDemux2(): BrickDigitalBase(3, 4) {codeName_ = "BrickDigitalDemux2"; type = name_ = "Demux2"; inNames[1] = "A0"; inNames[2] = "A1";}
	virtual BrickBase * copy() {return new BrickDigitalDemux2(*this);}
	virtual bool tick_body(double time);
};


class BrickDigitalDemux3: public BrickDigitalBase {
public:
	BrickDigitalDemux3(): BrickDigitalBase(4, 8) {codeName_ = "BrickDigitalDemux3"; type = name_ = "Demux3"; inNames[1] = "A0"; inNames[2] = "A1"; inNames[3] = "A2";}
	virtual BrickBase * copy() {return new BrickDigitalDemux3(*this);}
	virtual bool tick_body(double time);
};


class BrickDigitalDemux4: public BrickDigitalBase {
public:
	BrickDigitalDemux4(): BrickDigitalBase(5, 16) {codeName_ = "BrickDigitalDemux4"; type = name_ = "Demux4"; inNames[1] = "A0"; inNames[2] = "A1"; inNames[3] = "A2"; inNames[4] = "A3";}
	virtual BrickBase * copy() {return new BrickDigitalDemux4(*this);}
	virtual bool tick_body(double time);
};


class BrickDigitalDigitalToAnalog2: public BrickDigitalBase {
public:
	BrickDigitalDigitalToAnalog2(): BrickDigitalBase(3, 1) {codeName_ = "BrickDigitalDigitalToAnalog2"; type = name_ = "DigitalToAnalog2"; inNames[2] = "Max"; outNames[0] = "Output"; inputs[2] = 1.;}
	virtual BrickBase * copy() {return new BrickDigitalDigitalToAnalog2(*this);}
	virtual bool tick_body(double time) {outputs[0] = inputs[2] * (dbool(inputs[0]) + 2. * dbool(inputs[1])) / 3.; return true;}
};


class BrickDigitalDigitalToAnalog4: public BrickDigitalBase {
public:
	BrickDigitalDigitalToAnalog4(): BrickDigitalBase(5, 1) {codeName_ = "BrickDigitalDigitalToAnalog4"; type = name_ = "DigitalToAnalog4"; inNames[4] = "Max"; outNames[0] = "Output"; inputs[4] = 1.;}
	virtual BrickBase * copy() {return new BrickDigitalDigitalToAnalog4(*this);}
	virtual bool tick_body(double time);
};


class BrickDigitalDigitalToAnalog8: public BrickDigitalBase {
public:
	BrickDigitalDigitalToAnalog8(): BrickDigitalBase(9, 1) {codeName_ = "BrickDigitalDigitalToAnalog8"; type = name_ = "DigitalToAnalog8"; inNames[8] = "Max"; outNames[0] = "Output"; inputs[8] = 1.;}
	virtual BrickBase * copy() {return new BrickDigitalDigitalToAnalog8(*this);}
	virtual bool tick_body(double time);
};


class BrickDigitalAnalogToDigital2: public BrickDigitalBase {
public:
	BrickDigitalAnalogToDigital2(): BrickDigitalBase(2, 2) {codeName_ = "BrickDigitalAnalogToDigital2"; type = name_ = "AnalogToDigital2"; inNames[0] = "Input"; inNames[1] = "Max"; inputs[1] = 1.;}
	enum Inputs {Input, Max};
	virtual BrickBase * copy() {return new BrickDigitalAnalogToDigital2(*this);}
	virtual bool tick_body(double time);
};


class BrickDigitalAnalogToDigital4: public BrickDigitalBase {
public:
	BrickDigitalAnalogToDigital4(): BrickDigitalBase(2, 4) {codeName_ = "BrickDigitalAnalogToDigital4"; type = name_ = "AnalogToDigital4"; inNames[0] = "Input"; inNames[1] = "Max"; inputs[1] = 1.;}
	enum Inputs {Input, Max};
	virtual BrickBase * copy() {return new BrickDigitalAnalogToDigital4(*this);}
	virtual bool tick_body(double time);
};


class BrickDigitalAnalogToDigital8: public BrickDigitalBase {
public:
	BrickDigitalAnalogToDigital8(): BrickDigitalBase(2, 8) {codeName_ = "BrickDigitalAnalogToDigital8"; type = name_ = "AnalogToDigital8"; inNames[0] = "Input"; inNames[1] = "Max"; inputs[1] = 1.;}
	enum Inputs {Input, Max};
	virtual BrickBase * copy() {return new BrickDigitalAnalogToDigital8(*this);}
	virtual bool tick_body(double time);
};

#endif // BRICK_DIGITAL_H
