#include "brick_interface.h"


bool BrickInterfaceBaseIn::received(void * t, char * d, int s) {
	mutex.lock();
	BrickInterfaceBaseIn * i = (BrickInterfaceBaseIn * )t;
	memcpy(i->data.data(), d + i->header_, i->data.size() - i->header_);
	i->struct_.readData(i->data.data());
	mutex.unlock();
	//cout << "received " << s->data.size() << " bytes" << endl;
	return true;
}


bool BrickInterfaceBaseIn::tick_body(double time) {
	mutex.lock();
	for (uint i = 0; i < struct_.count(); ++i)
		outputs[i + 1] = struct_[i].value();
	mutex.unlock();
	return true;
}


bool BrickInterfaceBaseOut::tick_body(double time) {
	mutex.lock();
	for (uint i = 0; i < struct_.count(); ++i)
		struct_[i].setValue(inputs[i]);
	struct_.writeData(data.data());
	mutex.unlock();
	return true;
}


void BrickInterfaceSerialIn::started() {
	parameterChanged(0);
	struct_.writeData(data.data());
	header_ = parameters[5].toInt();
	ser.terminate();
	ser.setDevice(parameters[1].toString());
	PIFlags<PISerial::Parameters> f = 0;
	if (parameters[2].toBool()) f |= PISerial::ParityControl;
	if (parameters[3].toBool()) f |= PISerial::TwoStopBits;
	ser.setParameters(f);
	ser.setSpeed((PISerial::Speed)parameters[4].toInt());
	ser.setSlot(received);
	ser.setData(this);
	ser.setReadData(data.data(), header_, data.size() - header_);
	//cout << "header " << header_ << " bytes, data " << data.size() - header_ << " bytes" << endl;
	ser.start();
}


bool BrickInterfaceSerialIn::tick_body(double time) {
	outputs[0] = dbool(ser.initialized());
	return BrickInterfaceBaseIn::tick_body(time);
}


void BrickInterfaceSerialOut::started() {
	parameterChanged(0);
	ser.terminate();
	ser.setDevice(parameters[1].toString());
	PIFlags<PISerial::Parameters> f = 0;
	if (parameters[2].toBool()) f |= PISerial::ParityControl;
	if (parameters[3].toBool()) f |= PISerial::TwoStopBits;
	ser.setParameters(f);
	ser.setSpeed((PISerial::Speed)parameters[4].toInt());
	ser.init();
}


bool BrickInterfaceSerialOut::tick_body(double time) {
	BrickInterfaceBaseOut::tick_body(time);
	ser.send(data.data(), data.size());
	outputs[0] = dbool(ser.initialized());
	return true;
}


void BrickInterfaceUDPIn::started() {
	parameterChanged(0);
	eth.terminate();
	eth.setReadAddress(parameters[1].toString(), parameters[2].toInt());
	eth.setSlot(received);
	eth.setData(this);
	eth.start();
}


bool BrickInterfaceUDPIn::tick_body(double time) {
	outputs[0] = dbool(eth.receiverInitialized());
	return BrickInterfaceBaseIn::tick_body(time);
}


bool BrickInterfaceUDPOut::tick_body(double time) {
	BrickInterfaceBaseOut::tick_body(time);
	if (eth.send(parameters[1].toString(), parameters[2].toInt(), data.data(), data.size()))
		outputs[0] = 1.;
	else outputs[0] = 0.;
	return true;
}


void BrickInterfaceBinFileOut::started() {
	if (file.isOpened()) file.close();
	file.open(parameters[1].toString());
	if (!file.isOpened()) file.open(parameters[1].toString(), PIFile::Read | PIFile::Write | PIFile::New);
	if (parameters[3].toBool() > 0) file.clear();
	file.seekToEnd();
	id = parameters[2].toInt();
}


bool BrickInterfaceBinFileOut::tick_body(double time) {
	BrickInterfaceBaseOut::tick_body(time);
	file.writeBinary(id).writeBinary((ushort)struct_.size()).writeData(data.data(), struct_.size());
	if (parameters[4].toBool() > 0) file.flush();
	return true;
}

