#ifndef BRICK_LOGIC_H
#define BRICK_LOGIC_H

#include "brick_base.h"


class BrickLogicBase: public BrickBase {
public: BrickLogicBase(int inputs_num = 1, int outputs_num = 1, int parameters_num = 0): BrickBase(inputs_num, outputs_num, parameters_num) {lib = "Logics";}
};


class BrickLogicNot: public BrickLogicBase {
public:
	BrickLogicNot(): BrickLogicBase(1, 1) {codeName_ = "BrickLogicNot"; type = name_ = "Not";}
	virtual BrickBase * copy() {return new BrickLogicNot(*this);}
	virtual bool tick_body(double time) {outputs[0] = (inputs[0] > 0. ? 0. : 1.); return true;}
};


class BrickLogicAnd: public BrickLogicBase {
public:
	BrickLogicAnd(int inputs_num = 2): BrickLogicBase(inputs_num, 1) {codeName_ = "BrickLogicAnd"; type = name_ = "And"; inNames[0] = ""; io_Type = BrickBase::VariableInputs;}
	virtual BrickBase * copy() {return new BrickLogicAnd(*this);}
	virtual bool tick_body(double time);
};


class BrickLogicOr: public BrickLogicBase {
public:
	BrickLogicOr(int inputs_num = 2): BrickLogicBase(inputs_num, 1) {codeName_ = "BrickLogicOr"; type = name_ = "Or"; inNames[0] = ""; io_Type = BrickBase::VariableInputs;}
	virtual BrickBase * copy() {return new BrickLogicOr(*this);}
	virtual bool tick_body(double time);
};


class BrickLogicXor: public BrickLogicBase {
public:
	BrickLogicXor(int inputs_num = 2): BrickLogicBase(inputs_num, 1) {codeName_ = "BrickLogicXor"; type = name_ = "Xor"; inNames[0] = ""; io_Type = BrickBase::VariableInputs;}
	virtual BrickBase * copy() {return new BrickLogicXor(*this);}
	virtual bool tick_body(double time);
};


class BrickLogicNAnd: public BrickLogicBase {
public:
	BrickLogicNAnd(int inputs_num = 2): BrickLogicBase(inputs_num, 1) {codeName_ = "BrickLogicNAnd"; type = name_ = "NAnd"; inNames[0] = ""; io_Type = BrickBase::VariableInputs;}
	virtual BrickBase * copy() {return new BrickLogicNAnd(*this);}
	virtual bool tick_body(double time);
};


class BrickLogicNOr: public BrickLogicBase {
public:
	BrickLogicNOr(int inputs_num = 2): BrickLogicBase(inputs_num, 1) {codeName_ = "BrickLogicNOr"; type = name_ = "NOr"; inNames[0] = ""; io_Type = BrickBase::VariableInputs;}
	virtual BrickBase * copy() {return new BrickLogicNOr(*this);}
	virtual bool tick_body(double time);
};


class BrickLogicNXor: public BrickLogicBase {
public:
	BrickLogicNXor(int inputs_num = 2): BrickLogicBase(inputs_num, 1) {codeName_ = "BrickLogicNXor"; type = name_ = "NXor"; inNames[0] = ""; io_Type = BrickBase::VariableInputs;}
	virtual BrickBase * copy() {return new BrickLogicNXor(*this);}
	virtual bool tick_body(double time);
};


class BrickLogicMemory: public BrickLogicBase {
public:
	BrickLogicMemory(): BrickLogicBase(2, 1) {codeName_ = "BrickLogicMemory"; type = name_ = "Memory"; inNames[1] = "Write";}
	virtual BrickBase * copy() {return new BrickLogicMemory(*this);}
	enum Inputs {Input, Write};
	virtual bool tick_body(double time) {if (inputs[Write] > 0) outputs[0] = inputs[Input]; return true;}
};


class BrickLogicCompare: public BrickLogicBase {
public:
	BrickLogicCompare(double tolerance = 0.): BrickLogicBase(3, 3) {codeName_ = "BrickLogicCompare"; setTolerance(tolerance); type = name_ = "Compare"; inNames[0] = ""; inNames[2] = "Tolerance"; outNames[0] = "<"; outNames[1] = "="; outNames[2] = ">";}
	virtual BrickBase * copy() {return new BrickLogicCompare(*this);}
	enum Inputs {Tolerance = 2};
	void setTolerance(double tolerance) {inputs[Tolerance] = tolerance;}
	virtual bool tick_body(double time);
};

#endif // BRICK_LOGIC_H
