#ifndef BRICK_STATISTIC_H
#define BRICK_STATISTIC_H

#include "brick_math.h"


class BrickStatisticBase: public BrickBase {
public: BrickStatisticBase(int inputs_num = 1, int outputs_num = 1, int parameters_num = 0): BrickBase(inputs_num, outputs_num, parameters_num) {lib = "Statistic";}
};


class BrickStatisticMinMaxI: public BrickStatisticBase {
public:
	BrickStatisticMinMaxI(): BrickStatisticBase(2, 2) {codeName_ = "BrickStatisticMinMaxI"; type = name_ = "MinMax_Immediate"; outNames[Min] = "Min"; outNames[Max] = "Max"; inNames[0] = "0"; io_Type = VariableInputs;}
	enum Inputs {Min, Max};
	virtual BrickBase * copy() {return new BrickStatisticMinMaxI(*this);}
	virtual void reset_specified() {min = max = 0.;}
	virtual bool tick_body(double time);
private:
	double min, max;
};


class BrickStatisticMinMaxF: public BrickStatisticBase {
public:
	BrickStatisticMinMaxF(): BrickStatisticBase(1, 2) {codeName_ = "BrickStatisticMinMaxF"; type = name_ = "MinMax_Function"; outNames[Min] = "Min"; outNames[Max] = "Max";}
	enum Inputs {Min, Max};
	virtual BrickBase * copy() {return new BrickStatisticMinMaxF(*this);}
	virtual void reset_specified() {min = max = 0.;}
	virtual bool tick_body(double time) {if (min > inputs[0]) min = inputs[0]; if (max < inputs[0]) max = inputs[0]; outputs[Min] = min; outputs[Max] = max; return true;}
private:
	double min, max;
};


class BrickStatisticExpectation: public BrickStatisticBase {
public:
	BrickStatisticExpectation(): BrickStatisticBase(1, 1) {codeName_ = "BrickStatisticExpectation"; type = name_ = "Expectation";}
	virtual BrickBase * copy() {return new BrickStatisticExpectation(*this);}
	virtual void reset_specified() {i.reset(); pt = t = 0.;}
	virtual bool tick_body(double time);
private:
	BrickMathIntegral i; double pt, t;
};


class BrickStatisticVariance: public BrickStatisticBase {
public:
	BrickStatisticVariance(): BrickStatisticBase(1, 1) {codeName_ = "BrickStatisticVariance"; type = name_ = "Variance";}
	virtual BrickBase * copy() {return new BrickStatisticVariance(*this);}
	virtual void reset_specified() {i0.reset();  i1.reset(); pt = t = 0.;}
	virtual bool tick_body(double time);
private:
	BrickMathIntegral i0, i1; double pt, t;
};

#endif // BRICK_STATISTIC_H
